#!/usr/local/gis/bin/python
# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2005 Christian Mauduit <ufoot@ufoot.org>

import re
import sys
import cgi
import _pg
import mapscript

#print "Content-Type: text/plain"
#print 
#print "OK"

# Not really any way to avoid the fact that somewhere in
# the CGI we need to put a reference to a conf file or
# something on the filesystem...
PYGPSWEB_CONF_FILE="/usr/local/gis/etc/pygpsweb/cgi-bin.conf"

class Config:
    """Reads the config file from disk and stores values
    for further use. Beware this is not the same file than
    the one used for mod_python."""
    def __init__(self):
        """Config file is parsed when constructing this object."""
        cf=open(PYGPSWEB_CONF_FILE)
        regex=re.compile("^\s*(\w*)\s*\=\s*(.*)$")
        for i in cf.xreadlines():
            search=regex.search(i)
            if search:
                self.__dict__[search.groups()[0]]=search.groups()[1].rstrip()

class Db:
    """This is a simple wrapper class to wrap calls
    to database. Using such a wrapper allows us to
    put various callbacks, logs, and ultimately change
    the module used to access the database. It's more or less a
    duplicate of what's in the Db module of the mod_python
    code but copying it frees us from a dependency."""
    def __init__(self,name,host,port,login,password):
        """Constructor connects directly to the
        database."""
        self.cnx=_pg.connect(name,host,int(port),user=login,passwd=password)
    def query(self,str):
        """Sends a request to the PostGreSQL server."""
        return self.cnx.query(str)
    def query_dict(self,str):
        """Sends a request to the PostGreSQL server, and if the request
        returns a result, it sends the result as a list of dictionnaries."""
        res=None
        rs=self.query(str)
        if rs:
            res=rs.dictresult()
        if not res:
            # res to [] so that function always returns a list
            res=[]
        return res

class Map:
    """The main class we instanciate to generate the whole map."""
    def __init__(self,cnf):
        """Constructor of the map object, requires config to be passed."""
        self.config=cnf
        self.db=None
    def get_db(self):
        """Returns a Db handler, using this wrapper avoids
        creating Db connections for nothing. db member isn't
        initialized by constructor."""
        if not self.db:
            self.db=Db(self.config.db_name,
                       self.config.db_host,
                       self.config.db_port,
                       self.config.db_user,
                       self.config.db_passwd)
        return self.db
    def check_range(self,key,range_min,range_max):
        """Checks a form value for min/max boundaries."""
        self.form[key]=max(int(self.form[key]),range_min)
        self.form[key]=min(int(self.form[key]),range_max)
    def read_form(self):
        """Read all form parameters using the CGI interface."""
        form=cgi.FieldStorage()
        self.form={"map_width_px":320,
                   "map_height_px":240}
        for key in self.form:
            if form.has_key(key):
                self.form[key]=form.getvalue(key)
        if ((not form.has_key("long_min")) or
            (not form.has_key("long_max")) or
            (not form.has_key("lat_min")) or
            (not form.has_key("lat_max"))):
            self.default_area()
        else:
            for key in ["long_min","long_max","lat_min","lat_max"]:
                self.form[key]=float(form.getvalue(key))
        self.check_range("map_width_px",16,800)
        self.check_range("map_height_px",16,800)
    def default_area(self):
        """Calculates the default area when the area isn't correctly
        defined by the form parameters."""
        area=self.get_db().query_dict("SELECT min(xmin(the_geom)) AS long_min ,max(xmax(the_geom)) AS long_max, min(ymin(the_geom)) AS lat_min, max(ymax(the_geom)) AS lat_max FROM pygpsweb_raster")
        for key in ["long_min","long_max","lat_min","lat_max"]:
            self.form[key]=float(area[0][key])
    def setup_raster_layer(self):
        """Setup of the raster layer, ie make it point to the right
        data. We use the script to do this to avoid putting too many
        parameters in the MapServer config file."""
        raster=getLayerByName("raster")
    def calc_map(self):
        """Performs all the calcultations to create the UMN MapServer
        Map object."""
        self.map=mapscript.mapObj(self.config.default_map)
        self.map.set_width(self.form["map_width_px"])
        self.map.set_height(self.form["map_height_px"])
        self.map.setExtent(self.form["long_min"],
                           self.form["lat_min"],
                           self.form["long_max"],
                           self.form["lat_max"])
        #self.map.setExtent(-5,40,10,55)
    def write(self):
        """Writes the map on stdout, ie to the brower. Calls all necessary
        functions before, that is construct and call write(), and that's
        it."""
        self.read_form()
        self.calc_map()
        image=self.map.draw()
        print "Content-Type: image/png"
        print
        image.write(sys.stdout)

map=Map(Config())
map.write()

