# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2005 Christian Mauduit <ufoot@ufoot.org>

import random

from itools.xml import parser

import common

SERVICES_AVAILABLE=[
    "AddWaypoint",
    "DelWaypoint",
    "SetWaypoint",
    "AddTrackPoint",
    "DelTrackPoint",
    "SetTrackPoint",
    "AddRoutePoint",
    "DelRoutePoint",
    "SetRoutePoint",
    "DeleteLastWaypoint"]

class ServiceSoap(common.Common):
    """Handles namespace for the various soap services."""
    def __init__(self,context):
        """Initialize namespace from request parameters."""
        common.Common.__init__(self,context)
        self.context=context
        # Initialize all services entries to None so that
        # stl can test them for existence in stl:if attributes.
        for serv in SERVICES_AVAILABLE:
            self.ns[serv]=None
        service=""
        soap_input=self.context.request.read()
        soap_params={}
        # Using low-level XML parser in a non-bullet proof manner.
        # Point is itools handler was longer to set up, knowing
        # that there's no "file" (be it virtual...) that holds the
        # content, only an opened handler which is readable "a la"
        # stdin. Any idea welcome.
        for event, value, line_number in parser.parse(soap_input):
            if event == parser.START_ELEMENT:
                namespace, prefix, local_name = value
                if service:
                    param_name=local_name
                if local_name in SERVICES_AVAILABLE:
                    service=local_name
            if event == parser.END_ELEMENT:
                namespace, prefix, local_name = value
                if local_name == service:
                    break
            if event == parser.TEXT:
                if param_name:
                    soap_params[param_name]=value
        self.context.log("SOAP Request method=%s, params=%s" % (service,str(soap_params)))
        self.context.auth.init(user_id=soap_params["user_id"],
                               user_passwd=soap_params["user_passwd"])
        self.context.auth.check() # security, forbid anonymous users
        self.ns[service]={"Response":{}};
        if service == "AddWaypoint":
            result=self.add_waypoint()
        elif service == "DelWaypoint":
            result=self.del_waypoint()
        elif service == "SetWaypoint":
            result=self.set_waypoint()
        elif service == "AddTrackPoint":
            result=self.add_track_point()
        elif service == "DelTrackPoint":
            result=self.del_track_point()
        elif service == "SetTrackPoint":
            result=self.set_track_point()
        elif service == "AddRoutePoint":
            result=self.add_route_point(soap_params["Name"],
                                        float(soap_params["X"].replace(",",".")),
                                        float(soap_params["Y"].replace(",",".")),
                                        float(soap_params["Z"].replace(",",".")),
                                        int(soap_params["RouteId"]))
        elif service == "DelRoutePoint":
            result=self.del_route_point()
        elif service == "SetRoutePoint":
            result=self.set_route_point()
        elif service == "DeleteLastWaypoint":
            result=self.delete_last_waypoint()
        self.ns[service]["Response"]["Result"]=str(result);
        self.context.log("SOAP Response namespace=%s" % (self.ns[service]["Response"]))

    def get_file_from_route(self,route_id):
        """Utility function to get the file associated to a route."""
        rec=self.context.db.query_dict("SELECT file_id FROM pygpsweb_file, pygpsweb_route WHERE route_file=file_id AND route_id=%d" % route_id)
        return int(rec[0]["file_id"])
        
    def add_route_point(self,name,x,y,z,route_id):
        """Adds a new route point, ie creates a new waypoint and affects it to a route."""
        file_id=self.get_file_from_route(route_id)
        self.context.db.query("INSERT INTO pygpsweb_waypoint (waypoint_gpsid, waypoint_description, waypoint_geom, waypoint_file) VALUES ('%s', 'this is a test point 1', GeomFromText('POINT(%f %f %f)'), %d);" % (str(random.random())[-6:],x,y,z,file_id))
        # Ugly, need to make a PGSQL function for that...
        rec=self.context.db.query_dict("SELECT max(waypoint_id) AS new_id FROM pygpsweb_waypoint")
        new_id=int(rec[0]["new_id"])
        rec=self.context.db.query_dict("SELECT max(routepoint_order)+1 AS new_order FROM pygpsweb_routepoint WHERE routepoint_route=%d" % route_id)
        try:
            new_order=int(rec[0]["new_order"])
        except TypeError:
            new_order=1
        self.context.db.query("INSERT INTO pygpsweb_routepoint (routepoint_route, routepoint_waypoint, routepoint_order) VALUES (%d,%d,%d);" % (route_id,new_id,new_order))
        return 1
    def delete_last_waypoint(self):
        """Quick hack function to allow editing of my round-paris-crazy-tour while PyGpsWeb isn't finished."""
        rec=self.context.db.query_dict("SELECT max(waypoint_id) AS max_waypoint FROM pygpsweb_waypoint")
        try:
            max_waypoint=int(rec[0]["max_waypoint"])
        except TypeError:
            max_waypoint=1        
        self.context.db.query("DELETE FROM pygpsweb_waypoint WHERE waypoint_id=%d;" % max_waypoint)
        
        
        
